<div class="field-group">
  {{ field.label_tag }}
  {% if field.errors %}
    <div class="field-errors">
      {% for error in field.errors %}
        {{ error }}
      {% endfor %}
    </div>
  {% endif %}
  {{ field }}
  {% if field.help_text %}
    <small>{{ field.help_text }}</small>
  {% endif %}
</div>
