# SPDX-License-Identifier: MIT

"""The application's interfaces that are used to connect the different components.

Notes
-----
This package's code is not really specific to the Django framework. It is an
abstraction layer.

Primary focus is the provision of a plugin API, that allows the app to be
extendable with third-party applications.
"""
