{% for hidden_field in form.hidden_fields %}
{{ hidden_field }}
{% endfor %}

{% if form.non_field_errors %}
<div class="non-field-errors">
  {% for error in form.non_field_errors %}
    {{ error }}
  {% endfor %}
</div>
{% endif %}

{% for field in form.visible_fields %}
<div class="field-group">
  {{ field.label_tag }}
  {% if field.errors %}
    <div class="field-errors">
      {% for error in field.errors %}
        {{ error }}
      {% endfor %}
    </div>
  {% endif %}
  {{ field }}
  {% if field.help_text %}
    <small>{{ field.help_text }}</small>
  {% endif %}
</div>
{% endfor %}
