{% extends "calingen/app_base.html" %}

{% block page_title %}Event: EventUpdateView{% endblock page_title %}

{% block main %}
<h2>Update Event</h2>

<form method="post" novalidate class="calingen-form">
  {% csrf_token %}

  {% include "calingen/includes/form.html" with form=form %}

  <button type="submit" class="submit">Update Event</button>
  <button type="reset" class="cancel">Cancel</button>
</form>
{% endblock main %}
