# SPDX-License-Identifier: MIT

"""App-specific implementations of :class:`django.forms.Form` and related stuff like fields and widgets.

Warnings
--------
Siblings of :class:`django.forms.ModelForm` are (by definition) semantically
related to the respective implementation of :class:`django.db.models.Model`.

Thus, the specific forms for the app's models are included within the
:py:mod:`calingen.models` package.
"""
