# Generated by Django 4.1.5 on 2023-01-06 13:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calingen', '0001_initial'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='event',
            unique_together={('profile', 'title', 'start')},
        ),
    ]
