# SPDX-License-Identifier: MIT

"""Related but optional functionality to django-calingen.

This package includes working example implementations of
:class:`~calingen.interfaces.plugin_api.EventProvider`,
:class:`~calingen.interfaces.plugin_api.LayoutProvider` and
:class:`~calingen.interfaces.plugin_api.CompilerProvider`.

All of these implementation are fully working and may be used in a real project
setup of **django-calingen**.

They are derived from the author's own requirements and may not fully meet all
users' expectations.

However, they may be used as a reference for implementations of the app's
plugin interfaces (see :ref:`calingen-dev-doc-plugins-label` for the relevant
part of this documentation).
"""
