{% extends "calingen/app_base.html" %}

{% block page_title %}Event: EventDeleteView{% endblock page_title %}

{% block main %}
<form method="post" class="calingen-form">{% csrf_token %}
  <p>Are you sure you want to delete this event?</p>
  <p>
    <strong>{{ event_item.title }}</strong>: {{ event_item.timestamp|date:"c" }} - {{ event_item.get_category_display }}
  </p>
  <button type="submit" class="danger">Yes, I want to delete this Event!</button>
</form>
{% endblock main %}
